package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.IotConstants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.wx.domain.NotifyEventType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.common.utils.onenet.Token;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.vo.AttachVo;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.core.notification.Notification;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.Refund;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import com.wechat.pay.java.service.refund.model.Status;
import lombok.SneakyThrows;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;


/**
 * 订单支付回调 CallbackService 实现类
 */
@Service
public class CallbackServiceImpl implements CallbackService {

    private static final Logger logger = LoggerFactory.getLogger(CallbackServiceImpl.class);

    @Autowired
    private IEtOrderService orderService;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private IAsUserService userService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private NotificationParser notificationParser;

    @Autowired
    private IEtRefundService etRefundService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtCapitalFlowService etCapitalFlowService;

    @Value("${et.handlingCharge}")
    private String handlingCharge;

    @Autowired
    private ISysUserService sysUserService;


    /**
     * 微信支付回调
     */
    @Override
    @Transactional
    @SneakyThrows
    public void weChat(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        logger.info("【微信支付回调】接收对象 : " + JSON.toJSONString(body));
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);

        // 支付成功通知
        if (NotifyEventType.TRANSACTION_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 Transaction
            Transaction transaction = checkAndParse(request, body, Transaction.class);
            if (Transaction.TradeStateEnum.SUCCESS.equals(transaction.getTradeState())) {
                // 充值成功后的业务处理
                AttachVo attachVo = JSONObject.parseObject(transaction.getAttach(),AttachVo.class);
                logger.info("【微信支付回调】附加信息 : " + JSON.toJSONString(attachVo));
                String outTradeNo = transaction.getOutTradeNo();
                EtOrder order = orderService.selectEtOrderByOutTradeNo(outTradeNo);
                logger.info("【微信支付回调】订单信息 : " + JSON.toJSONString(order));

                AsUser asUser = userService.selectUserById(order.getUserId());
                /** 支付回调逻辑  1. 处理预约还是开锁  电压 */
                AsDevice asDevice = null;
                if(StrUtil.isNotBlank(order.getSn())){
                    asDevice = asDeviceService.selectAsDeviceBySn(order.getSn());
                }
                String iotToken = Token.getToken();
                //先判断是骑行订单还是押金，如果是骑行订单
                // 还要区分是取消预约支付
                //      如果是,更新订单状态，pay_time、pay_type（默认是wx）、appointment_fee等于total_fee，状态改为4 订单结束
                // 如果是扫码骑行后还车支付
                //      更新订单状态，pay_time、pay_type（默认是wx）、total_fee等于预约费+骑行费+管理费+调度费，状态改为4 订单结束，保存微信内部订单号 callbackVo.getOutTradeNo()
                order.setPaid("1");
                order.setPayTime(DateUtils.getNowDate());
                order.setPayType(ServiceConstants.PAY_TYPE_WX);
                if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_RIDING)){
                    logger.info("【微信支付回调】骑行支付");
                    // 1-骑行支付  关锁
                    order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
                    order.setMark("骑行支付");
                    asDeviceService.sendCommand(asDevice.getMac(), iotToken, IotConstants.COMMAND_OPEN,"套餐开锁");
                    asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//还车后车辆正常运营
                    asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);

                    // 新增资金流水记录
                    capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_INCOME,ServiceConstants.ORDER_TYPE_RIDING);
                    // 还车结算___小时后自动退押金---创建一个定时器TimerTask，计算出退还时间后，执行退款操作
                    EtFeeRule rule = etFeeRuleService.selectEtFeeRuleByRuleId(order.getRuleId());
                    Integer autoRefundDeposit = rule.getAutoRefundDeposit();
                    if(autoRefundDeposit!=null && autoRefundDeposit>0){
                        //创建一个定时器TimerTask，计算出退还时间后，执行退款操作
                        Timer timer = new Timer();
                        TimerTask task = new TimerTask() {
                            @Override
                            public void run() {
                                // 退款
                                Long userId = order.getUserId();
                                EtOrder etOrder = new EtOrder();
                                etOrder.setUserId(userId);
                                etOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
                                etOrder.setType(ServiceConstants.ORDER_TYPE_DEPOSIT);
                                etOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
                                List<EtOrder> etOrders = orderService.selectEtOrderList(etOrder);
                                if (etOrders.size() == 0 || ObjectUtil.isNull(etOrders) ) {
                                    throw new ServiceException("押金充值记录不存在");
                                }
                                Optional<EtOrder> latestOrder = etOrders.stream()
                                        .max(Comparator.comparing(EtOrder::getPayTime));
                                if (latestOrder.isPresent()) {
                                    EtOrder newestOrder = latestOrder.get();
                                    // 处理找到的最新支付时间的订单
                                    Long areaId = order.getAreaId();
                                    EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
                                    String deposit = area.getDeposit();
                                    if(newestOrder.getTotalFee().compareTo(new BigDecimal(deposit))!=0){
                                        throw new ServiceException("押金充值记录与当前运营区的押金不同");
                                    }
                                    Refund refund = wxPayService.refund(newestOrder,autoRefundDeposit+"个小时后自动退款", newestOrder.getTotalFee());
                                    EtRefund refund1= orderService.createRefund(etOrder, newestOrder.getTotalFee(), null, null, null, null, refund,ServiceConstants.REFUND_TYPE_DEPOSIT);
                                    int i = etRefundService.insertEtRefund(refund1);
                                    if(i>0){
                                        logger.info("【自动退款】保存退款对象成功");
                                    }

                                    // 新增资金流水记录
                                    capitalFlowRecords(newestOrder,ServiceConstants.FLOW_TYPE_DISBURSE,ServiceConstants.ORDER_TYPE_DEPOSIT_REFUND);
                                } else {
                                    throw new ServiceException("没有找到押金充值记录");
                                }
                            }
                        };
                        // 往后推autoRefundDeposit小时执行
                        Date refundTime = DateUtils.getTimeAfterXHours(order.getPayTime(), autoRefundDeposit);
                        timer.schedule(task, refundTime);
                    }
                }else if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_APPOINTMENT)){
                    logger.info("【微信支付回调】取消预约支付");
                    // 2-取消预约支付
                    order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
                    order.setMark("取消预约支付");
                    asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//取消预约支付后车辆正常运营
                    asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
                }else if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_MEAL)){//废弃
                    logger.info("【微信支付回调】套餐支付");
                    // 3-套餐支付  套餐支付中分为预约车辆和立即开锁骑行
                    if(attachVo.getIsAppointment()){//购买套餐后预约
                        order.setStatus(ServiceConstants.ORDER_STATUS_IN_APPOINTMENT);
                        order.setAppointmentStartTime(DateUtils.getNowDate());
                        order.setMark("套餐预约支付");
                        /** 2.发送命令*/
                        asDeviceService.sendCommand(asDevice.getMac(), iotToken,IotConstants.COMMAND_CLOSE,"套餐预约");
                        /** 3.更新车辆状态*/
                        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT);//预约中
                        asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
                        logger.info("【微信支付回调】套餐预约成功");
                    }else{
                        order.setMark("套餐骑行支付");
                        order.setStatus(ServiceConstants.ORDER_STATUS_RIDING);//骑行中
                        /** 2.发送命令*/
                        asDeviceService.sendCommand(asDevice.getMac(), iotToken,IotConstants.COMMAND_OPEN,"套餐开锁");
                        /** 3.更新车辆状态*/
                        asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
                        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_USING);
                    }
                    EtFeeRule etFeeRule = etFeeRuleService.selectEtFeeRuleByRuleId(order.getRuleId());
                    if(ObjectUtil.isNull(etFeeRule)){
                        throw new ServiceException("套餐不存在");
                    }
//                    Integer time = etFeeRule.getTime();//时间：以小时为单位
//                    //当前时间往后推time个小时
//                    Date ruleEndTime = DateUtils.addHours(DateUtils.getNowDate(),time);
//                    order.setRuleEndTime(ruleEndTime);
                }else if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_DEPOSIT)){
                    logger.info("【微信支付回调】押金支付");
                    // 4-押金支付
                    order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
                    asUser.setBalance(asUser.getBalance().add(order.getTotalFee()));
                    order.setMark("押金支付");

                    // 新增资金流水记录
                    capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_INCOME,ServiceConstants.ORDER_TYPE_DEPOSIT);
                    // 删除用户缓存
                    String token = attachVo.getToken();
                    logger.info("【微信支付回调】删除用户缓存:"+token);
                    if (StringUtils.isNotNull(token))
                    {
                        redisCache.deleteObject(CacheConstants.LOGIN_TOKEN_KEY + token);
                    }
                }else{
                    logger.error("【微信支付回调】 : 支付场景不存在");
                }
                if(ObjectUtil.isNotNull(asDevice)){
                    int device = asDeviceService.updateAsDevice(asDevice);
                    if(device==0){
                        logger.error("【微信支付回调】更新车辆状态失败");
                        throw new ServiceException("【微信支付回调】更新车辆状态失败");
                    }
                }
                int updateEtOrder = orderService.updateEtOrder(order);
                if(updateEtOrder==0){
                    logger.error("【微信支付回调】更新订单信息失败");
                    throw new ServiceException("【微信支付回调】更新订单信息失败");
                }
                int updateUser = userService.updateUser(asUser);
                if(updateUser==0){
                    logger.error("【微信支付回调】更新用户押金失败");
                    throw new ServiceException("【微信支付回调】更新用户押金失败");
                }
            }
        }
    }

    /**
     * 资金流水记录
     * */
    private void capitalFlowRecords(EtOrder order,String type,String busType) {
        EtCapitalFlow capitalFlow = new EtCapitalFlow();
        capitalFlow.setAreaId(order.getAreaId());
        capitalFlow.setOrderNo(order.getOrderNo());
        capitalFlow.setOutTradeNo(order.getOutTradeNo());
        capitalFlow.setType(type);
        capitalFlow.setBusType(busType);
        capitalFlow.setAmount(order.getTotalFee());
        if(busType.equals(ServiceConstants.ORDER_TYPE_DEPOSIT) || busType.equals(ServiceConstants.ORDER_TYPE_DEPOSIT_REFUND)){
            capitalFlow.setOperatorBalance(BigDecimal.ZERO);
            capitalFlow.setOperatorDividend(BigDecimal.ZERO);
            capitalFlow.setPartnerDividend(BigDecimal.ZERO);
        }else{
            //获取所有合伙人列表
            SysUser sysUser = new SysUser();
            sysUser.setUserType("03");
            List<SysUser> sysUsers = sysUserService.selectUserList(sysUser);
            double totalDividendProportion = sysUsers.stream()
                    .mapToDouble(SysUser::getDividendProportion)
                    .sum();//算出总的分成比例
            BigDecimal decimal = new BigDecimal(totalDividendProportion).divide(new BigDecimal(100), 2, BigDecimal.ROUND_HALF_UP);
            BigDecimal partnerDividend = order.getTotalFee().multiply(decimal);
            BigDecimal operatorDividend = order.getTotalFee().subtract(capitalFlow.getPartnerDividend());
            if(type.equals(ServiceConstants.FLOW_TYPE_INCOME)){
                capitalFlow.setPartnerDividend(partnerDividend);
                capitalFlow.setOperatorDividend(operatorDividend);
//           todo  capitalFlow.setOperatorBalance();
            }else{
                capitalFlow.setPartnerDividend(partnerDividend.negate());
                capitalFlow.setOperatorDividend(operatorDividend.negate());
//                todo  capitalFlow.setOperatorBalance();
            }
        }
        BigDecimal bigDecimal = new BigDecimal(handlingCharge);
        capitalFlow.setHandlingCharge(bigDecimal.multiply(order.getTotalFee()));//手续费
        capitalFlow.setPayType(ServiceConstants.PAY_TYPE_WX);
        capitalFlow.setCreateTime(DateUtils.getNowDate());

        int i = etCapitalFlowService.insertEtCapitalFlow(capitalFlow);
        if(i==0){
            throw new ServiceException("保存资金流水记录失败");
        }
    }

    /**
     * 微信退款回调
     */
    @Override
    @Transactional
    public void weChatRefund(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        logger.info("【微信退款回调】接收对象 : " + JSON.toJSONString(body));
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);
        logger.info("【微信退款回调】转换成notification: " + JSON.toJSONString(notification));
        // 退款成功通知
        if (NotifyEventType.REFUND_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 RefundNotification
            RefundNotification refundNotification = checkAndParse(request, body, RefundNotification.class);
            logger.info("【微信退款回调】转换成RefundNotification: " + JSON.toJSONString(refundNotification));
            if (Status.SUCCESS.equals(refundNotification.getRefundStatus())) {
                // 退款成功后  改变 etRefundService
                String outRefundNo = refundNotification.getOutRefundNo();
                EtRefund etRefund = new EtRefund();
                etRefund.setRefundNo(outRefundNo);
                etRefund.setRefundResult(Constants.SUCCESS2);
                int i = etRefundService.updateEtRefundByRefundNo(etRefund);
                if(i==0){
                    logger.error("【微信退款回调】更新退款单失败");
                    throw new ServiceException("【微信退款回调】更新退款单失败");
                }
                EtRefund refund = etRefundService.selectEtRefundByRefundNo(outRefundNo);
                EtOrder order = orderService.selectEtOrderByOrderNo(refund.getOrderNo());
                // 新增资金流水记录
                capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_DISBURSE,ServiceConstants.ORDER_TYPE_RIDING_REFUND);
            }
        }
    }

    /**
     * 验签并解析
     * @param request 请求
     * @param body 请求体
     * @param clazz 返回值类型
     */
    private <T> T checkAndParse(HttpServletRequest request, String body, Class<T> clazz) {
        // 构造 RequestParam
        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(request.getHeader("Wechatpay-Serial"))
                .nonce(request.getHeader("Wechatpay-Nonce"))
                .signature(request.getHeader("Wechatpay-Signature"))
                .timestamp(request.getHeader("Wechatpay-Timestamp"))
                .body(body)
                .build();
        // 验签
        return notificationParser.parse(requestParam, clazz);
    }

}
