package com.ruoyi.system.service.impl;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.config.WxPayConfig;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.EtOrder;
import com.ruoyi.system.domain.vo.AttachVo;
import com.ruoyi.system.service.IAsUserService;
import com.ruoyi.system.service.IEtOperatingAreaService;
import com.ruoyi.system.service.IEtOrderService;
import com.ruoyi.system.service.IWxPayService;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.jsapi.JsapiService;
import com.wechat.pay.java.service.payments.jsapi.JsapiServiceExtension;
import com.wechat.pay.java.service.payments.jsapi.model.*;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.RefundService;
import com.wechat.pay.java.service.refund.model.AmountReq;
import com.wechat.pay.java.service.refund.model.CreateRequest;
import com.wechat.pay.java.service.refund.model.Refund;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;

/**
 * 微信支付服务
 * @author 辉
 * 2024/3/11
 */
@Service
@Slf4j
public class WxPayService implements IWxPayService {

    @Autowired
    private JsapiService jsapiService;

    @Autowired
    private JsapiServiceExtension jsapiServiceExtension;

    @Autowired
    private WxPayConfig wxPayConfig;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IAsUserService asUserService;

    @Autowired
    private NotificationParser notificationParser;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    public RefundService refundService2;

    @Autowired
    private RedisLock redisLock;

    private static final String CNY = "CNY";

    private static final String PREPAY_LOCK = "prepay:";

    @Override
    public PrepayWithRequestPaymentResponse prepayWithRequestPayment(String payType,EtOrder order) {
        String billNo = order.getOrderNo();
        // 创建订单
        AsUser user = asUserService.selectUserById(order.getUserId());
        try {
            // 获取JSAPI所需参数
            PrepayRequest request = new PrepayRequest();
            request.setAmount(getAmount(order.getTotalFee()));
            String outTradeNo = IdUtils.getOrderNo("wx");
            order.setOutTradeNo(outTradeNo);
            int updateEtOrder = etOrderService.updateEtOrder(order);
            if(updateEtOrder == 0){
                throw new ServiceException("更新订单outTradeNo失败");
            }
            request.setOutTradeNo(outTradeNo);
            request.setAppid(wxPayConfig.getAppId());
            request.setMchid(wxPayConfig.getMerchantId());
            String type = order.getType();
            String description = type.equals(ServiceConstants.ORDER_TYPE_RIDING) ? "骑行订单-"+billNo : "押金充值-"+billNo;
            LoginUser loginUser = SecurityUtils.getLoginUser();
            log.info("【预下单】获取登录用户信息:"+JSON.toJSONString(loginUser));
            request.setAttach(JSON.toJSONString(new AttachVo(payType,user.getUserId(), loginUser.getToken())));
            request.setDescription(description);
            request.setNotifyUrl(wxPayConfig.getNotifyUrl());
            request.setPayer(getPayer(user.getWxopenid()));
            PrepayWithRequestPaymentResponse res = jsapiServiceExtension.prepayWithRequestPayment(request);
            return res;
        } finally {
            redisLock.unlock(PREPAY_LOCK + billNo);
        }
    }

    /**
     * 关闭支付订单
     * @param billNo 平台订单编号
     */
    @Override
    public void closeOrder(String billNo) {
        CloseOrderRequest request = new CloseOrderRequest();
        request.setMchid(wxPayConfig.getMerchantId());
        request.setOutTradeNo(billNo);
        jsapiService.closeOrder(request);
    }

    @Override
    public Transaction queryOrderById(String prePayId) {
        QueryOrderByIdRequest request = new QueryOrderByIdRequest();
        request.setMchid(wxPayConfig.getMerchantId());
        request.setTransactionId(prePayId);
        return jsapiService.queryOrderById(request);
    }

    @Override
    public Transaction queryOrderByOutTradeNo(String billNo) {
        QueryOrderByOutTradeNoRequest request = new QueryOrderByOutTradeNoRequest();
        request.setMchid(wxPayConfig.getMerchantId());
        request.setOutTradeNo(billNo);
        return jsapiService.queryOrderByOutTradeNo(request);
    }

    /**
     * 退款
     * @param etOrder 原订单对象
     * @param reason 退款原因
     * @param amount 退款金额
     */
    @Override
    public Refund refund(EtOrder etOrder,String reason,BigDecimal amount) {
        CreateRequest request = new CreateRequest();
        request.setOutTradeNo(etOrder.getOutTradeNo());
        request.setOutRefundNo(IdUtils.randomUUID2());
        request.setReason(reason);
        request.setAmount(getAmountReq(etOrder.getTotalFee(),amount));
        request.setNotifyUrl(wxPayConfig.getRefundNotifyUrl());
        log.info("【退款】请求微信参数:【{}】",JSON.toJSONString(request));
        return refundService2.create(request);
    }

//    /**
//     * 微信支付通知
//     * @param request 请求
//     */
//    @Override
//    @Transactional
//    public void payNotify(HttpServletRequest request) {
//
//        String body = HttpUtils.getBody(request);
//        // 解析通知数据
//        Notification notification = JSON.parseObject(body, Notification.class);
//
//        // 判断是否重复通知，重复通知则忽略
//        if (isRepeatNotify(notification.getId())) {
//            return;
//        }
//
//        // 支付成功通知
//        if (NotifyEventType.TRANSACTION_SUCCESS.getValue().equals(notification.getEventType())) {
//            // 验签、解密并转换成 Transaction
//            Transaction transaction = checkAndParse(request, body, Transaction.class);
//
//            if (Transaction.TradeStateEnum.SUCCESS.equals(transaction.getTradeState())) {
//                SmTransactionBill bill = transactionBillService.selectSmTransactionBillByBillNo(transaction.getOutTradeNo());
//                ServiceUtil.assertion(bill == null, "订单不存在");
//
//                // 充值成功，修改订单状态
//                transactionBillService.rechargeSuccess(bill.getBillId(), DateUtils.getNowDate());
//                // 保存通知数据
//                saveNotifyData(bill.getBillId(), notification, transaction);
//            }
//        }
//
//    }

//    /**
//     * 通知是否重复
//     * @param notifyId 通知id
//     */
//    private boolean isRepeatNotify(String notifyId) {
//        SmWxPayNotify repeat = smWxPayNotifyMapper.selectSmWxPayNotifyByNotifyId(notifyId);
//        return repeat != null;
//    }

//    /**
//     * 保存通知数据
//     * @param billId 订单id
//     * @param body  请求体
//     * @param plaintext 明文
//     */
//    private void saveNotifyData(long billId, Notification body, Object plaintext) {
//        SmWxPayNotify data = new SmWxPayNotify();
//        data.setNotifyId(body.getId());
//        data.setBillId(billId);
//        data.setBody(JSON.toJSONString(body));
//        data.setPlaintext(JSON.toJSONString(plaintext));
//        int i = smWxPayNotifyMapper.insertSmWxPayNotify(data);
//        ServiceUtil.assertion(i != 1, "保存通知数据失败");
//    }

    /**
     * 验签并解析
     * @param request 请求
     * @param body 请求体
     * @param clazz 返回值类型
     */
    private <T> T checkAndParse(HttpServletRequest request, String body, Class<T> clazz) {
        // 构造 RequestParam
        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(request.getHeader("Wechatpay-Serial"))
                .nonce(request.getHeader("Wechatpay-Nonce"))
                .signature(request.getHeader("Wechatpay-Signature"))
                .timestamp(request.getHeader("Wechatpay-Timestamp"))
                .body(body)
                .build();
        // 验签
        return notificationParser.parse(requestParam, clazz);
    }

    private Payer getPayer(String openId) {
        Payer payer = new Payer();
        payer.setOpenid(openId);
        return payer;
    }

    private Amount getAmount(BigDecimal money) {
        Amount amount = new Amount();
        amount.setTotal(money.multiply(new BigDecimal(100)).intValue());
        amount.setCurrency(CNY);
        return amount;
    }

    private AmountReq getAmountReq(BigDecimal total,BigDecimal rerundAmount) {
        AmountReq amount = new AmountReq();
        amount.setTotal(total.multiply(new BigDecimal(100)).longValue());
        amount.setRefund(rerundAmount.multiply(new BigDecimal(100)).longValue());
        amount.setCurrency(CNY);
        return amount;
    }


}
